<?php
/**
 * @brief		Member Sync
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	(DP41) Referrals System
 * @since		28 Feb 2017
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\rsystem\extensions\core\MemberSync;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Member Sync
 */
class _rsystemMemberSync
{
	/**
	 * Member account has been created
	 *
	 * @param	$member	\IPS\Member	New member account
	 * @return	void
	 */
	public function onCreateAccount( $member )
	{
        /* Update user account */
        \IPS\Db::i()->update( 'core_members', array( 'dp3_rs_hashcode' => \IPS\rsystem\Settings::getHashCode( $member->email ) ), array( array( "member_id = ?", $member->member_id ) ) );
        
        /* Is enabled? */
        if( \IPS\rsystem\Settings::nexusIntegrationIsEnabled() && ! $member->dp3_rs_referred_by )
        {
    		if ( \IPS\Dispatcher::hasInstance() and \IPS\Dispatcher::i()->controllerLocation === 'front' and isset( \IPS\Request::i()->cookie['referred_by'] ) )
    		{
    			$referredBy = \IPS\Member::load( \IPS\Request::i()->cookie['referred_by'] );
    			if ( $referredBy->member_id )
    			{
    				/* Key */				
    				$key = md5( $_SERVER['REMOTE_ADDR'] . ')(&^$#!%^&E$^*)' . uniqid( microtime(), true ) );
    				
    				/* Add log */				
                    $log               = new \IPS\rsystem\Log;
                    $log->time         = time();
                    $log->inviter_id   = $referredBy->member_id;
                    $log->invited_id = $member->member_id;
                    $log->secure_key   = $key; 
                    $log->status       = 'used';
                    $log->times_sent   = 1;
                    $log->invited_ip   = $member->ip_address;
                    $log->save();
                    
                    /* Save the referral info */
                    \IPS\Db::i()->update( 'core_members', array( 'dp3_rs_referred_by' => $referredBy->member_id ), array( 'member_id = ?', $member->member_id ) );  
    			}
    		}            
        }
	}
	
	/**
	 * Member has validated
	 *
	 * @param	\IPS\Member	$member		Member validated
	 * @return	void
	 */
	public function onValidate( $member )
	{
        /* Play with check key */
		if( ( \IPS\Settings::i()->dp41_rsystem_enable && ! \IPS\Settings::i()->dp41_rsystem_minimum_posts ) || ( \IPS\Settings::i()->dp41_rsystem_minimum_posts && ( \IPS\Settings::i()->dp41_rsystem_minimum_posts <= $member->member_posts ) ) )
		{
			\IPS\rsystem\Settings::playWithCheckKey();
		}
	}
	
	/**
	 * Member has logged on
	 *
	 * @param	\IPS\Member	$member		Member that logged in
	 * @return	void
	 */
	public function onLogin( $member )
	{
    
	}
	
	/**
	 * Member has logged out
	 *
	 * @param	\IPS\Member		$member			Member that logged out
	 * @param	\IPS\Http\Url	$redirectUrl	The URL to send the user back to
	 * @return	void
	 */
	public function onLogout( $member, $returnUrl )
	{
	
	}
	
	/**
	 * Member account has been updated
	 *
	 * @param	$member		\IPS\Member	Member updating profile
	 * @param	$changes	array		The changes
	 * @return	void
	 */
	public function onProfileUpdate( $member, $changes )
	{
	
	}
	
	/**
	 * Member is flagged as spammer
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onSetAsSpammer( $member )
	{
		
	}
	
	/**
	 * Member is unflagged as spammer
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onUnSetAsSpammer( $member )
	{
		
	}
	
	/**
	 * Member is merged with another member
	 *
	 * @param	\IPS\Member	$member		Member being kept
	 * @param	\IPS\Member	$member2	Member being removed
	 * @return	void
	 */
	public function onMerge( $member, $member2 )
	{
		/* Get actual referrer */
		try
        {
            $act = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( "i_invited_id = ?", $member2->member_id ) )->first();
    		if( $act['i_inviter_id'] != $member->member_id )
    		{			
    			/* Add points */			
                \IPS\rsystem\Settings::addPoints( $member->member_id );
    		}
        }
        catch ( \UnderflowException $e )
        {}
		
	    /* Update */
        \IPS\Db::i()->update( 'rsystem_referrals', array( 'i_inviter_id' => $member->member_id ), array( array( "i_inviter_id = ? AND i_inviter_id <> ?", $member2->member_id, $member->member_id ) ) );	
	}
	
	/**
	 * Member is deleted
	 *
	 * @param	$member	\IPS\Member	The member
	 * @return	void
	 */
	public function onDelete( $member )
	{		
		/* Find transaction to delete */
		try
        {
            $data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( "i_invited_id = ? OR i_user_pending = ?", $member->member_id, $member->member_id ) )->first();
            \IPS\rsystem\Settings::addPoints( $data->i_inviter_id, true );
            
            $log = \IPS\rsystem\Log::load( $data['i_id'] );
            $log->delete();
        }
        catch ( \UnderflowException $e )
        {}
        		
		/* Delete all transactions where the mid is found */
        \IPS\Db::i()->delete( 'rsystem_referrals', array( 'i_inviter_id = ?', $member->member_id ) );
        \IPS\Db::i()->update( 'core_members', array( 'dp3_rs_referred_by' => 0 ), array( array( "member_id = ?", $member->member_id ) ) );
	}

	/**
	 * Email address is changed
	 *
	 * @param	\IPS\Member	$member	The member
	 * @param 	string		$new	New email address
	 * @param 	string		$old	Old email address
	 * @return	void
	 */
	public function onEmailChange( $member, $new, $old )
	{

	}

	/**
	 * Password is changed
	 *
	 * @param	\IPS\Member	$member	The member
	 * @param 	string		$new	New password
	 * @return	void
	 */
	public function onPassChange( $member, $new )
	{

	}
}