//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class rsystem_hook_rsystemContentTrick extends _HOOK_CLASS_
{
	/**
	 * Referrals
	 *
	 * @return	string
	 */
	protected function rsystem()
	{
		try
		{
			try
			{
				if( ! \IPS\rsystem\Settings::checkAccess() )
		        {
		            \IPS\Output::i()->error( 'rsystem_cant_access', 'RSYSTEM_001', 403, '' );
		        }
		        
		        $html = '';       
		        $html = self::getLogs();
		
				/* Build form */
				$form = new \IPS\Helpers\Form();
		        $form->class = 'ipsForm_vertical ipsPad';
		        
		        $form->add( new \IPS\Helpers\Form\Stack( 'rsystem_invites_emails', NULL, TRUE, array( 'maxItems' => \IPS\Settings::i()->dp41_rsystem_invites_invites_per_session ), function( $val )
					{
						if( is_array( $val ) )
						{
							$val = array_unique( $val );
		                    
		            		if( ! count( $val ) )
		            		{
		                        throw new \DomainException( \IPS\Member::loggedIn()->language()->get( 'rsystem_not_any_mails_found' ) );		
		            		}
		                    
		                    foreach ( $val as $name )
							{
								if ( \IPS\Member::load( $name, 'name' )->member_id )
								{
		                            throw new \DomainException( \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_invites_emails_member_exists', FALSE, array( 'sprintf' => array( $name ) ) ) );	
								}
		    					if ( filter_var( $name, FILTER_VALIDATE_EMAIL ) === FALSE )
		    					{
		    						throw new \DomainException( sprintf( \IPS\Member::loggedIn()->language()->get( 'import_email_invalid' ), $name ) );
		    					}
		                        $test = \IPS\rsystem\Log::load( $name, 'friend_mail' );
		            
		            			if( $test->id )
		            			{
		            				throw new \DomainException( sprintf( \IPS\Member::loggedIn()->language()->get( 'rsystem_invites_emails_referral_exists' ), $name ) );
		            			}
							}
						}
					}
		            , NULL, NULL, 'rsystem_invites_emails' ) );
		        
		        if( \IPS\Settings::i()->dp41_rsystem_invites_custom_message )
		        {    
		            $form->add( new \IPS\Helpers\Form\TextArea( 'rsystem_invite_message', \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_invite_message_default' ), TRUE, array( 'rows' => 3, 'class' => 'ipsField_codeInput'  ), NULL, NULL, NULL, 'rsystem_invite_message' ) );
		        }
		        
				if ( $values = $form->values() )
				{
		     		/* Check day limit */    		
		    		$dtime	= time() - ( 60 * 60 * 24 );
		            $count  = \IPS\Db::i()->select( 'COUNT(*) as count', 'rsystem_referrals', array( 'i_time >= ' . $dtime . ' AND i_inviter_id = ' . \IPS\Member::loggedIn()->member_id ) )->first();
		
		    		/* Check day limit */   		
		    		if( \IPS\Settings::i()->dp41_rsystem_invites_day_limit <= $count ) 
		    		{
						$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_referrals_limit_day_expired' );
						\IPS\Output::i()->output = $form;
						return;   
		    		}
		
		    		/* Add mails to the SQL */
		    		foreach( $values['rsystem_invites_emails'] as $mailOk )
		    		{
		                $log              = new \IPS\rsystem\Log;
		                $log->time        = time();
		                $log->inviter_id  = \IPS\Member::loggedIn()->member_id;
		                $log->friend_mail = $mailOk;
		                $log->body        = isset( $values['rsystem_invite_message'] ) ? \IPS\Text\Parser::parseStatic( $values['rsystem_invite_message'] ) : '';
		                #$log->secure_key  = md5( $mailOk . ')(&^$#!%^&E$^*)' ); 
		                $log->status      = 'sent';
		                $log->times_sent  = 1;
		                $log->save();
		                
		                $log->secure_key = md5( $mailOk . ')(&^$#!%^&E$^*)' . $log->id );
		                $log->save();
		    			
		    			/* Send mail */   			
		    			\IPS\rsystem\Settings::sendMail( $log );
		            }
		        
					\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=rsystem' ), 'rsystem_invites_sent' );
		            return;
				}
				
				$inviteForm = \IPS\Theme::i()->getTemplate( 'global', 'core', 'admin' )->block( 'rsystem_invites', $form, FALSE );
		        
		        /* Prepare message */
		        $refLink = '';
		        if( \IPS\Settings::i()->rsystem_referral_link_show_mode == 'main' )
		        {
		            $refLink = self::getReferralUrl();
		            $form->addMessage( \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_ucp_settings_referral_link', NULL, array( 'sprintf' => array( $refLink ) ) ), 'ipsMessage ipsMessage_info' );
		        }
		
				/* Set Session Location */
				\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=rsystem', NULL, '' ), array(), 'loc_viewing_rsystem' );
				
				\IPS\Output::i()->breadcrumb[] = array( NULL, \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_ucp_title' ) );
				\IPS\Output::i()->title        = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_ucp_title' );
				\IPS\Output::i()->output       = \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' )->referralsUcp( $html, \IPS\Settings::i()->dp41_rsystem_enable_invites ? $inviteForm : FALSE, $refLink );
			}
			catch ( \RuntimeException $e )
			{
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
    
    protected function getRefferedNexusStuff()
    {
	try
	{
		try
		{
		        /* Enabled? */
				if( ! \IPS\rsystem\Settings::nexusIntegrationIsEnabled() )
		        {
		            \IPS\Output::i()->error( 'rsystem_cant_access', 'RSYSTEM_028', 403, '' );
		        }
		        
		        /* Some init */
		        $where   = array();
		        $where[] = 'referred_by = ' . \IPS\Member::loggedIn()->member_id; 
		        
		        $referrals = new \IPS\Helpers\Table\Db( 'nexus_referrals', \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=getRefferedNexusStuff' ), implode( ' AND ', $where ) );
				$referrals->langPrefix = '';
		        $referrals->title = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_nexus_title' );
		        $referrals->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' ), 'indexReferredNexusList' );
		        $referrals->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' ), 'indexReferredNexusRow' );    
				$referrals->include = array( 'member_id', 'amount' );
				$referrals->sortBy = $referrals->sortBy ?: 'member_id';
				$referrals->parsers = array(
					'member_id'	=> function( $v )
					{
						try
						{
							return \IPS\nexus\Customer::load( $v )->link();
						}
						catch ( \OutOfRangeException $e )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'deleted_member' );
						}
					},
					'email'	=> function( $v, $row )
					{
						try
						{
							return \IPS\Member::load( $row['member_id'] )->email;
						}
						catch ( \OutOfRangeException $e )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'deleted_member' );
						}
					},
					'amount'	=> function( $v )
					{
						$return = array();
						if ( $v )
						{
							foreach ( json_decode( $v, TRUE ) as $currency => $amount )
							{
								$return[] = new \IPS\nexus\Money( $amount, $currency );
							}
						}
						else
						{
							$return[] = new \IPS\nexus\Money( 0, \IPS\nexus\Customer::loggedIn()->defaultCurrency() );
						}
						return implode( '<br>', $return );
					}
				);
				
		        /* Set Session Location */
				\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=getRefferedNexusStuff' ), array(), 'loc_viewing_rsystem_nexus_referred_logs' );
		        	
				/* Display */
		        \IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=getRefferedNexusStuff' ), \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_nexus_title' ) );
		        \IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_nexus_title' );
				\IPS\Output::i()->output = (string) $referrals;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
    
    protected function getLogs()
    {
	try
	{
		try
		{
				$data = array();
		        
		        /* Play with invites */	
				if( \IPS\Settings::i()->dp41_rsystem_enable_invites )
				{					
					/* Check day limit */
					$dtime	= time() - ( 60 * 60 * 24 );
					$count  = \IPS\Db::i()->select( 'COUNT(*) as count', 'rsystem_referrals', array( 'i_time >= ' . $dtime . ' AND i_inviter_id = ' . \IPS\Member::loggedIn()->member_id ) )->first();
					
					/* Add today invites */
					$data['today_counter'] = ( $count ) ? $count : 0;	
							
					/* Set limit data */
					$data['one_limit'] = \IPS\Settings::i()->dp41_rsystem_invites_invites_per_session;
					$data['day_limit'] = \IPS\Settings::i()->dp41_rsystem_invites_day_limit;			
				}
				
				/* Points integration is enabled? */
				if( \IPS\Settings::i()->rsystem_points_integration_points != '-1' )
				{
					/* Check system */			
			/*
            if( \IPS\rsystem\Settings::_checkSql( 'standard' ) )
			{
				if( \IPS\Settings::i()->rsystem_points_integration_table == 'members' )
				{					
					$cl = \IPS\Settings::i()->rsystem_points_integration_column;
                    $data['stored_points'] = \IPS\Member::loggedIn()->$cl;				
				}
			}
			else
			{				
				if( \IPS\rsystem\Settings::_checkSql( 'custom' ) )
				{
				    $act = \IPS\Db::i()->select( $this->column, $this->table, array( $this->mem_column_id . ' = ?', \IPS\Member::loggedIn()->member_id ) );					
                    $cl = \IPS\Settings::i()->rsystem_points_integration_column;	
					$data['stored_points'] = $act->$cl;			
				}				
			}
            */			
				}
		            
		        /* Play with table */
		        $sort    = 'i_time';
		        $order   = 'desc';                                            
		        $where   = array();
		        $title   = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_show_logs_table_title' );
		        
		        $where[] = 'i_inviter_id = ' . \IPS\Member::loggedIn()->member_id;        
		        
		        /* Create the table */
				$table = new \IPS\Helpers\Table\Db( 'rsystem_referrals', \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=rsystem', 'front', '' ), implode( ' AND ', $where ) );
				$table->title = $title;
		        
		        if( isset( $data['stored_points'] ) )
		        {
		            $table->title = $table->title . ' ' . \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_stored_points' ) . ": " . $data['stored_points']; 
		        } 
		        
		        $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' ), 'indexLogsList' );
		        $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' ), 'indexLogsRow' );        
		        $table->langPrefix    = '';
		
		        $table->limit = 15;
		        
		        /* Default sort options */
				$table->sortBy = $table->sortBy ?: $sort;
				$table->sortDirection = $table->sortDirection ?: $order;
		
		        /* Column stuff */
				$table->include = array( 'i_id', 'i_inviter_id', 'i_invited_id', '_i_invited_id', '_i_status', 'i_time', 'i_friend_mail', 'i_status', 'i_times_sent', 'i_user_pending', 'members_disable_pm' );
		        $table->noSort	= array( '' );
				$table->mainColumn = 'i_invited_id';
		
				/* Custom parsers */
				$table->parsers = array(
					'_i_invited_id' => function( $val, $row )
					{
		                return $row['i_invited_id'];
					}, 
		            
					'members_disable_pm' => function( $val, $row )
					{
		                $member = \IPS\Member::load( $row['i_invited_id'] );
		                return $member->members_disable_pm;
					}, 
		            
					'_i_status' => function( $val, $row )
					{
		                return $row['i_status'];
					}, 
		            
					'i_invited_id' => function( $val, $row )
					{
		                $member = \IPS\Member::load( $row['i_invited_id'] );
		                if( $member->member_id )
		                {
		                    $ret = "<a href='" . $member->url() . "'>" . htmlentities( $member->name, \IPS\HTMLENTITIES, 'UTF-8', FALSE ) . "</a>";
		                }
		                else
		                {
		                    $ret = $member->name;
		                }
		                
		                if( $row['i_friend_mail'] )
		                {
		                    $ret .= "<br /><span class='ipsType_small'>(" . $row['i_friend_mail'] . ")</span>";
		                }
		                return $ret;
					}, 
		            
					'i_time' => function( $val, $row )
					{
		                return \IPS\DateTime::ts( $val );
					},
		
					'i_status' => function( $val, $row )
					{
		    			/* Check expired status */	
		                if( \IPS\Settings::i()->dp41_rsystem_enable_invites && \IPS\Settings::i()->dp41_rsystem_invites_expire_time )
		                {
		        			if( ( ( $row['i_time'] + ( \IPS\Settings::i()->dp41_rsystem_invites_expire_time * 60 * 60 * 24 ) ) < time() ) && ( $row['i_status'] == 'sent' ) )
		        			{
		                        $log = \IPS\rsystem\Log::load( $row['i_id'] );
		                        $log->status = 'expired';
		                        $log->save(); 
		        			}
		                }            
		
		                /* Work with it... :) */            
						if( ( $row['i_status'] == 'sent' ) && $row['i_user_pending'] && ! $row['i_invited_id'] )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__pending' );
						}					
						elseif( $row['i_status'] == 'sent' )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__' . $val );
						}
						elseif( $row['i_status'] == 'used' )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__' . $val );				
						}
						elseif( $row['i_status'] == 'expired' )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__' . $val );				
						}				
						else
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__unknown' );	    
						}
					},                  
				); 
		        
		        /* Return */
		        return $table;         
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
    
    protected function getReferralUrl()
    {
	try
	{
		try
		{
		        $refLink = \IPS\Http\Url::internal( 'app=rsystem&reff=' . \IPS\Member::loggedIn()->member_id, 'front', '' );
		        
		 		if( \IPS\Settings::i()->dp41_rsystem_enable_mask )
				{
					$refLink = \IPS\Http\Url::internal( 'app=rsystem&reff=' . \IPS\Member::loggedIn()->dp3_rs_hashcode, 'front', '' );
				}
		        
		        return $refLink;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
    
    protected function rsystemUserSettings()
    {
	try
	{
		try
		{
				if( ! \IPS\rsystem\Settings::checkAccess() )
		        {
		            \IPS\Output::i()->error( 'rsystem_cant_access', 'RSYSTEM_002', 403, '' );
		        }
		
				/* Build form */
				$form = new \IPS\Helpers\Form();
		        $form->class = 'ipsForm_vertical ipsPad';
		        
		        if( \IPS\Settings::i()->rsystem_referral_link_show_mode == 'settings' )
		        {
		            $refLink = self::getReferralUrl();
		            $form->addMessage( \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_ucp_settings_referral_link', NULL, array( 'sprintf' => array( $refLink ) ) ), 'ipsMessage ipsMessage_info' );
		        }
		        
		        $form->add( new \IPS\Helpers\Form\YesNo( 'rsystem_ucp_settings_enable_collecting', \IPS\Member::loggedIn()->dp3_rs_menable ) );
		
				if ( $values = $form->values() )
				{
		            \IPS\Member::loggedIn()->dp3_rs_menable = $values['rsystem_ucp_settings_enable_collecting'];
		            \IPS\Member::loggedIn()->save(); 
		
					\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=rsystem' ), 'saved' );
				}
				
				/* Display */
		        \IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_ucp_title' );
				\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'global', 'core', 'admin' )->block( 'rsystem_user_settings', $form, FALSE );
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
    
    protected function rsystemResend()
    {
	try
	{
		try
		{
				/* Option is enabled? */		
				if( ! \IPS\Settings::i()->dp41_rsystem_enable_invites )
				{
					\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=rsystem' ) );
				}
				
				/* Internal init */
				$id	= intval( \IPS\Request::i()->id );
				
				/* Get invite information from DB */
				try
		        {
		            $data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( "i_status = 'sent' AND i_id = ?", $id ) )->first();
		            $log  = \IPS\rsystem\Log::load( $data['i_id'] );
		        }
		        catch ( \UnderflowException $e )
		        {}
		        
				/* Do we have anything? */
				if( ! $data['i_id'] )
				{
					\IPS\Output::i()->error( 'rsystem_error_cant_resend_this_invite', 'RSYSTEM_003', 403, '' );
				}
				
				/* Reached the resends limit? */
				if( \IPS\Settings::i()->dp41_rsystem_invites_resend_attempts < ( $log->times_sent + 1 ) )
				{
					\IPS\Output::i()->error( 'rsystem_error_cant_resend_this_invite', 'RSYSTEM_004', 403, '' );				
				}
		
				/* Still here? Resend! */		
				\IPS\rsystem\Settings::sendMail( $log, false );	
				
				/* Update counter */
				$log->times_sent = $log->times_sent + 1;
		        $log->save();
				
		        /* Redirect */
				\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=rsystem' ), 'rsystem_resend_resent' );
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
    
    protected function getRefferedAllNexusStuff()
    {
	try
	{
		try
		{
		        /* Enabled? */
				if( ! \IPS\rsystem\Settings::nexusIntegrationIsEnabled() )
		        {
		            \IPS\Output::i()->error( 'rsystem_cant_access', 'RSYSTEM_028', 403, '' );
		        }
		        
		        /* Not an admin? */
		        if( ! \IPS\Member::loggedIn()->isAdmin() )
		        {
		            \IPS\Output::i()->error( 'rsystem_cant_access', 'RSYSTEM_029', 403, '' );
		        }
		        
		        /* Build table */
		        $referrals = new \IPS\Helpers\Table\Db( 'nexus_referrals', \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=getRefferedAllNexusStuff' ) );
				$referrals->langPrefix = '';
		        $referrals->title = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_all_nexus_title' );
		        $referrals->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' ), 'indexReferredAllNexusList' );
		        $referrals->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'ucp', 'rsystem', 'front' ), 'indexReferredAllNexusRow' );    
				$referrals->include = array( 'referred_by', 'member_id', 'amount' );
				$referrals->sortBy = $referrals->sortBy ?: 'member_id';
				$referrals->parsers = array(
					'referred_by'	=> function( $v )
					{
						try
						{
							return \IPS\nexus\Customer::load( $v )->link();
						}
						catch ( \OutOfRangeException $e )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'deleted_member' );
						}
					},
					'member_id'	=> function( $v )
					{
						try
						{
							return \IPS\nexus\Customer::load( $v )->link();
						}
						catch ( \OutOfRangeException $e )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'deleted_member' );
						}
					},
					'email'	=> function( $v, $row )
					{
						try
						{
							return \IPS\Member::load( $row['member_id'] )->email;
						}
						catch ( \OutOfRangeException $e )
						{
							return \IPS\Member::loggedIn()->language()->addToStack( 'deleted_member' );
						}
					},
					'amount'	=> function( $v )
					{
						$return = array();
						if ( $v )
						{
							foreach ( json_decode( $v, TRUE ) as $currency => $amount )
							{
								$return[] = new \IPS\nexus\Money( $amount, $currency );
							}
						}
						else
						{
							$return[] = new \IPS\nexus\Money( 0, \IPS\nexus\Customer::loggedIn()->defaultCurrency() );
						}
						return implode( '<br>', $return );
					}
				);
				
		        /* Set Session Location */
				\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=getRefferedAllNexusStuff' ), array(), 'loc_viewing_rsystem_all_nexus_referred_logs' );
		        	
				/* Display */
		        \IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=core&module=system&controller=content&do=getRefferedAllNexusStuff' ), \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_all_nexus_title' ) );
		        \IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_all_nexus_title' );
				\IPS\Output::i()->output = (string) $referrals;
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
}
