<?php


namespace IPS\rsystem\modules\admin\logs;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * manage
 */
class _manage extends \IPS\Dispatcher\Controller
{	
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'manage_manage' );
		parent::execute();
        
        \IPS\Output::i()->output .= \IPS\Application::load( 'rsystem' )->cpr();
	}
	
	/**
	 * Manage
	 *
	 * @return	void
	 */
	protected function manage()
	{		
		/* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'rsystem_referrals', \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage' ) );
        $table->langPrefix = 'rsystem_log_';
        $table->limit = 20;

		/* Column stuff */
		$table->include    = array( 'i_id', 'i_invited_id', 'i_inviter_id', 'i_status', 'i_time', 'i_invited_ip' );
        $table->noSort	   = array( 'i_id', 'i_invited_ip' );
		$table->mainColumn = 'i_invited_id';
        
		$table->widths  = array(
            'i_invited_id'  => 22,
            'i_inviter_id'  => 22,
            'i_status'      => 13,
            'i_time'        => 13,
            'i_invited_ip'  => 13,
		);

        $table->filters = array( 
                    'rsystem_logs_status__sent'    => array( 'i_status=?', 'sent' ), 
                    'rsystem_logs_status__used'    => array( 'i_status=?', 'used' ),
                    'rsystem_logs_status__pending' => array( 'i_status=?', 'pending' ),
                    'rsystem_logs_status__expired' => array( 'i_status=?', 'expired' ),
        );      
		
        /* Default sort options */
		$table->sortBy = $table->sortBy ?: 'i_time';
		$table->sortDirection = $table->sortDirection ?: 'desc';
		
        /* Search */
		$table->quickSearch = 'i_invited_id';
		$table->advancedSearch = array(
			'i_invited_id'          => \IPS\Helpers\Table\SEARCH_MEMBER,
            'i_inviter_id'		    => \IPS\Helpers\Table\SEARCH_MEMBER,
            'i_invited_ip'          => \IPS\Helpers\Table\SEARCH_CONTAINS_TEXT,
            'i_time'                => \IPS\Helpers\Table\SEARCH_DATE_RANGE,           
		);

		/* Custom parsers */
		$table->parsers = array(
			'i_invited_id' => function( $val, $row )
			{
                $member = \IPS\Member::load( $val );
                
                if( $member->member_id )
                {
                    $ret = "<a href='" . $member->acpUrl() . "'>" . htmlentities( $member->name, \IPS\HTMLENTITIES, 'UTF-8', FALSE ) . "</a>";
                }
                else
                {
                    $ret = $row['i_friend_mail'] ? ( '<em>' . $row['i_friend_mail'] . '</em>' ) : $member->name;
                }
                
                if( ( $row['i_status'] == 'sent' ) && $row['i_user_pending'] )
                {
                    $ret .= '<br />(<em>' . \IPS\Member::load( $row['i_user_pending'] )->name . '</em>)';     
                }
                
                return $ret;
			},
			'i_inviter_id' => function( $val, $row )
			{
                $member = \IPS\Member::load( $val );
                return "<a href='" . $member->acpUrl() . "'>" . htmlentities( $member->name, \IPS\HTMLENTITIES, 'UTF-8', FALSE ) . "</a>";
			}, 
			'i_status'			=> function( $val, $row )
			{
				/* Check expired status */		
                if( \IPS\Settings::i()->dp41_rsystem_enable_invites && \IPS\Settings::i()->dp41_rsystem_invites_expire_time )
                {                                
    				if( ( ( $row['i_time'] + ( \IPS\Settings::i()->dp41_rsystem_invites_expire_time * 86400 ) ) < time() ) && ( $row['i_status'] == 'sent' ) )
    				{
                        $log = \IPS\rsystem\Log::load( $row['i_id'] );
                        $log->status = 'expired';
                        $log->save(); 
    				}
                }
                
                /* Work with it... :) */            
				if( ( $row['i_status'] == 'sent' ) && $row['i_user_pending'] && ! $row['i_invited_id'] )
				{
					return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__pending' );
				}					
				elseif( $row['i_status'] == 'sent' )
				{
					return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__' . $val );
				}
				elseif( $row['i_status'] == 'used' )
				{
					return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__' . $val );				
				}
				elseif( $row['i_status'] == 'expired' )
				{
					return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__' . $val );				
				}				
				else
				{
					return \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_status__unknown' );	    
				}
			}, 
            'i_invited_ip'			=> function( $val, $row )
            {
                return $val ? "<a href='" . \IPS\Http\Url::internal( "app=core&module=members&controller=ip&ip={$val}" ) . "'>{$val}</a>" : \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_none' );
            },                       
			'i_time'			=> function( $val, $row )
			{
                return \IPS\DateTime::ts( $val );
			},
		);
        
        $table->rowButtons = function( $row )
		{
			$return = array();

            if( ( $row['i_status'] == 'sent' ) )
            {
        		try
        		{
        			$pending = \IPS\Member::load( $row['i_user_pending'] );
                    
                    if( $pending->member_id )
                    {
                        $return['accept'] = array(
            				'title'		=> 'rsystem_logs_accept',
            				'icon'		=> 'check',
            				'link'		=> \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage&do=accept&id=' ) . $row['i_id'],
            			); 
                    }
        		}
        		catch( \OutOfRangeException $ex )
        		{ 
        		}
            }

            if( $row['i_inviter_id'] && ( $row['i_status'] != 'expired' ) )
            {
                $return['edit'] = array(
    				'title'		=> 'rsystem_logs_edit',
    				'icon'		=> 'pencil',
    				'link'		=> \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage&do=form&id=' ) . $row['i_id'],
    			); 
            }
            
            $return['delete'] = array(
            	'icon'		=> 'times-circle',
            	'title'		=> 'delete',
            	'link'		=> \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage&do=delete&id=' ) . $row['i_id'],
            	'data'      => array(
            		'delete' => '',
            		'delete-warning' => \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_delete_confirm_desc' )
            	)
            );
          
			return $return;
		};
        
        /* Root buttons */
		$table->rootButtons = array(
			'refresh'		=> array(
				'icon'		=> 'refresh',
				'title'		=> 'rsystem_logs_refresh',
				'link'		=> \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage&do=refresh' ),
				'data'		=> array(  'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_refresh' ) )
			),
			'add'		=> array(
				'icon'		=> 'plus',
				'title'		=> 'rsystem_logs_add',
				'link'		=> \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage&do=form' ),
				'data'		=> array( 'ipsDialog' => '', 'ipsDialog-title' => \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_add' ) )
			)
		);
        
		/* Display */
        \IPS\Output::i()->title     = \IPS\Member::loggedIn()->language()->addToStack( 'menu__rsystem_logs_manage' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'global', 'core' )->block( 'title', (string) $table );
	}
    
    public function form()
    {
        /* Load */
        try
		{
			$item = \IPS\rsystem\Log::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
		}
        
        /* Expired? */
        if( $item->status == 'expired' )
        {
            \IPS\Output::i()->error( 'log_item_cant_edit', 'RSYSTEM_027', 403, '' );
        }
        
        $current = NULL;
		
        if ( \IPS\Request::i()->id )
		{
            if( ! $item->id )
            {
                \IPS\Output::i()->error( 'log_item_not_found', 'RSYSTEM_005', 404, '' );    
            } 
            
            $current = $item;                     
		}

        /* Build the form */
		$form         = new \IPS\Helpers\Form();
        $formElements = \IPS\rsystem\Log::formElements( $current, NULL );
        
        $form->addHeader( 'rsystem_log_main_info' );
        $form->add( $formElements['log_form_invited_id'] );
        $form->add( $formElements['log_form_inviter_id'] );
        $form->add( $formElements['log_form_add_points'] );

        if ( $values = $form->values() )
		{		
            /* Check default values */
    		if( ! $values['log_form_invited_id']->member_id && empty( $current ) )
    		{ 
				$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_no_invited_member' );
				\IPS\Output::i()->output = $form;
				return;  
    		}
    		
    		if( ! $values['log_form_inviter_id']->member_id )
    		{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_no_referrer_member' );
				\IPS\Output::i()->output = $form;
				return;
    		}
    		
    		/* New transaction, but already invited? */   		
    		if( empty( $current ) && $values['log_form_invited_id']->dp3_rs_referred_by )
    		{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_already_invited' );
				\IPS\Output::i()->output = $form;
				return;
    		}
    		
    		/* The same? */
    		if( $values['log_form_invited_id']->member_id == $values['log_form_inviter_id']->member_id )
    		{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_the_same_account' );
				\IPS\Output::i()->output = $form;
				return;
    		}
    		
    		/* Referrer can't referr */
    		if( ! $values['log_form_inviter_id']->dp3_rs_menable )
    		{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_referrer_cant_refer' );
				\IPS\Output::i()->output = $form;
				return;
    		}
    		
    		/* Referrer is banned from this system? */   		
    		if( $values['log_form_inviter_id']->dp3_rs_banned )
    		{
				$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_referrer_is_banned' );
				\IPS\Output::i()->output = $form;
				return;
    		}
            
            /* New or? */
            $new = false;
			if ( empty( $current ) )
			{
                $new     = true;
				$current = \IPS\rsystem\Log::createFromForm( $values, NULL );
                
                /* Add points? */
                if( \IPS\Request::i()->log_form_add_points )
                {
                    /* Add points to referrer */
                    \IPS\rsystem\Settings::addPoints( $values['log_form_inviter_id']->member_id ); 
                }
                
                $current->status = 'used';
                
                $referredMember                     = \IPS\Member::load( $values['log_form_invited_id']->member_id );
                $referredMember->dp3_rs_referred_by = $values['log_form_inviter_id']->member_id;
                $referredMember->save();
                
                /* Try add nexus info */
    			if( \IPS\rsystem\Settings::nexusIntegrationIsEnabled() )
                {
                    if ( $referredMember->member_id )
        			{  
                        try
                		{
                			$existsReferreral = \IPS\Db::i()->select( '*', 'nexus_referrals', array( 'member_id=? AND referred_by=?', $referredMember->member_id, $referredMember->dp3_rs_referred_by ) )->first();
                		}
                		catch ( \UnderflowException $e )
                		{
                            \IPS\Db::i()->insert( 'nexus_referrals', array(
                                					'member_id'		=> $referredMember->member_id,
                                					'referred_by'	=> $referredMember->dp3_rs_referred_by,
                                					'amount'		=> '',
                            ) );
                		}
                        
        			}
                }
			}
            
			/* Just editing, thanks */
			if ( ! $new )
			{
				$current->processForm( $values );	
                
    			/* Referrer was changed? */    			
    			if( $current->inviter_id != $values['log_form_inviter_id']->member_id )
    			{
    				/* Play with points */
    				\IPS\rsystem\Settings::addPoints( $values['log_form_inviter_id']->member_id );
    				\IPS\rsystem\Settings::addPoints( $current->inviter_id, true );
    			
        			/* Update! */
                    $referredMember                     = \IPS\Member::load( $values['log_form_invited_id']->member_id );
                    $referredMember->dp3_rs_referred_by = $values['log_form_inviter_id']->member_id;
                    $referredMember->save(); 
                }
			}
            
			$current->save();
            
           \IPS\Output::i()->redirect( $current->acpUrl(), 'saved' );
        }   
        
        if ( \IPS\Request::i()->id )
		{
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_edit' );
		}
		else
		{
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_logs_add' );
		}    
        
        \IPS\Output::i()->output = $form;
    }
    
    public function delete()
    {
		/* Load item */
		try
		{
            $item = \IPS\rsystem\Log::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'log_item_not_found', 'RSYSTEM_006', 404, '' );
		}

		/* Make sure the user confirmed the deletion */
		\IPS\Request::i()->confirmedDelete();
		
        /* Check if this members was referred by someone */
		$invitedMember = \IPS\Member::load( $item->invited_id );
        
        if( $invitedMember->member_id && ! $invitedMember->dp3_rs_referred_by )
		{
            \IPS\Output::i()->error( 'rsystem_error_no_referred_info', 'RSYSTEM_007', 404, '' );			
		}
		
		/* Still here? Make the changes */
		if( $invitedMember->member_id )
		{
			/* Save member */
			$invitedMember->dp3_rs_referred_by = 0;
            $invitedMember->save();
            
			/* Remove also added points if exists */
			\IPS\rsystem\Settings::addPoints( $invitedMember->dp3_rs_referred_by, true );		
		}
        
		/* Delete */
		\IPS\Session::i()->log( 'acplog__rsystem_log_item_deleted' );
		$item->delete();
       
        /* View? */
        $url = $item->acpUrl();
   
		/* Boink */
		\IPS\Output::i()->redirect( $url, 'deleted' );
    }
    
    public function accept()
    {
		/* Load item */
		try
		{
            $item = \IPS\rsystem\Log::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'log_item_not_found', 'RSYSTEM_008', 404, '' );
		}

		/* Check status */
		if( ( $item->status != 'sent' ) || ! $item->user_pending )
		{
			\IPS\Output::i()->error( 'rsystem_error_wrong_transaction', 'RSYSTEM_009', 404, '' );		
		}
		
		/* Add and check promotion points */
		$s = \IPS\rsystem\Settings::handlePointsSystem( $item->inviter_id );
		
		if( ! $s && \IPS\Settings::i()->dp41_rsystem_minimum_posts )
		{
			\IPS\rsystem\Settings::addPoints( $item->inviter_id );
			
            $inviter = \IPS\Member::load( $item->inviter_id );
            $inviter->dp3_rs_padded = 1;
            $inviter->save();
		}
		
		/* Validate user */
		try
		{
			$member = \IPS\Member::load( $data->user_pending );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2C114/A', 404, 'RSYSTEM_010' );
		}
		
		$member->validationComplete();
		
		/* Log */
		\IPS\Session::i()->log( 'acplog__members_approved', array( $member->name => FALSE ) );
			
		/* Mark me */		
        $item->status = 'used';
        $item->invited_id = $member->member_id;
        $item->save();	
   
        /* Try add nexus info */
        if( \IPS\rsystem\Settings::nexusIntegrationIsEnabled() )
        {
    		$referredBy = \IPS\Member::load( $member->member_id );
    		if ( $referredBy->member_id )
    		{  
                try
        		{
        			$existsReferreral = \IPS\Db::i()->select( '*', 'nexus_referrals', array( 'member_id=? AND referred_by=?', $data['i_inviter_id'], $referredBy->member_id ) )->first();
        		}
        		catch ( \UnderflowException $e )
        		{
                    \IPS\Db::i()->insert( 'nexus_referrals', array(
                        					'member_id'		=> $data['i_inviter_id'],
                        					'referred_by'	=> $referredBy->member_id,
                        					'amount'		=> '',
                    ) );
        		}
                
    		}
        }
        
        /* View? */
        $url = $item->acpUrl();
   
		/* Boink */
		\IPS\Output::i()->redirect( $url, 'rsystem_accepted_log' );   
    }
    
	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function refresh()
	{
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_refreshing' );
        
        $perGo = 50;
        $baseUrl = \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage&do=refresh' );	

        \IPS\Output::i()->output = new \IPS\Helpers\MultipleRedirect(
			$baseUrl,
			function( $doneSoFar ) use ( $perGo )
			{					
                $select = \IPS\Db::i()->select( 'i_inviter_id, i_invited_id', 'rsystem_referrals', array( 'i_inviter_id > ? AND i_invited_id > ?', 0, 0 ), 'i_id DESC', array( $doneSoFar, $perGo ), NULL, NULL, \IPS\Db::SELECT_SQL_CALC_FOUND_ROWS );
    			$total	= $select->count( TRUE );
    
    			if ( ! $select->count() )
    			{
    				return NULL;
    			}

    			foreach( $select as $log )
    			{
                    if( $log['i_inviter_id'] && $log['i_invited_id'] )
                    {
                        \IPS\Db::i()->update( 'core_members', array( 'dp3_rs_referred_by' => $log['i_inviter_id'] ), array( 'member_id = ?', $log['i_invited_id'] ) );    
                    } 	
    			}
    
    			$doneSoFar += $perGo;
    			return array( $doneSoFar, \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referred_refreshing' ), ( 100 * $doneSoFar ) / $total );	
			},
			function()
			{
                \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=rsystem&module=logs&controller=manage' ), 'rsystem_refreshing_referred_finished' );
			}
		); 
    }
}