<?php


namespace IPS\rsystem\modules\front\rsystem;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * invite
 */
class _invite extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		if( ! isset( \IPS\Request::i()->do ) )
        {
            \IPS\Output::i()->error( 'rsystem_wrong_action', 'RSYSTEM_011', 403, '' );
        }
	}
	
	public function receiveValidate()
	{
		/* INIT */		
		$key = trim( \IPS\Request::i()->key );
			
		/* Get invite info from SQL */
		try
        {
            $data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( 'i_secure_key = ?', $key ) )->first(); 
		}
        catch ( \UnderflowException $e )
        {}
        
		/* No invite? */
		if( ! $data['i_id'] )
		{
			#Remove all cookies
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
            
            \IPS\Output::i()->error( 'rsystem_invite_no_such_key', 'RSYSTEM_012', 403, '' );				
		}
		
		/* Invite was expired? */
		if( \IPS\Settings::i()->dp41_rsystem_invites_expire_time )
		{
			if( ( $data['i_time'] + ( \IPS\Settings::i()->dp41_rsystem_invites_expire_time * ( 60 * 60 * 24 ) ) ) < time() )
			{
				#Remove all cookies
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
							
                \IPS\Output::i()->error( 'rsystem_invite_expired', 'RSYSTEM_013', 403, '' );				
			}
		}
		
		/* Used */
		if( $data['i_status'] != 'sent' )
		{
			#Remove all cookies
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
					
            \IPS\Output::i()->error( 'rsystem_invite_used_already', 'RSYSTEM_014', 403, '' );				
		}
        
        /* Pending? */
		if( $data['i_status'] == 'sent' )
		{					
#??         \IPS\Output::i()->error( 'rsystem_invite_is_pending', 'RSYSTEM_015', 403, '' );				
		}
		
		/* Check key if it's correct */
		if( $key != md5( $data['i_friend_mail'] . ')(&^$#!%^&E$^*)' . $data['i_id'] ) )
		{
			#Remove all cookies
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );

            \IPS\Output::i()->error( 'rsystem_invite_wrong_key', 'RSYSTEM_016', 403, '' );				
		}
		
		/* Set cookie */		
        \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', $key, \IPS\DateTime::create()->add( new \DateInterval( 'P1Y') ), FALSE );
        
		/* Redirect! */
		\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );		
	}

	public function checkRefferal()
	{
		/* INIT */
		$_reff	= trim( \IPS\Request::i()->reff );
        
        /* Get referral ID */
        try
        {
            $referrer = \IPS\Db::i()->select( 'member_id', 'core_members', array( 'dp3_rs_hashcode = ?', $_reff ) )->first(); 
        }
        catch ( \UnderflowException $e )
        {}
        
        if( ! $referrer )
        {
 			#Redirect!
			\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );	 	           
        }
        else
        {
            $reff = $referrer;
        }
		
		/* The same key as member ID */
		if( $reff == \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'rsystem_the_same_reff_key', 'RSYSTEM_017', 403, '' );
		}
		
		/* Logged member? */		
		if( \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'rsystem_logged_already', 'RSYSTEM_018', 403, '' );		
		}
		
		/* Check invite exists already */
		try
        {
            $check = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( 'i_invited_ip = ?', $_SERVER['REMOTE_ADDR'] ) )->first(); 
		}
        catch ( \UnderflowException $e )
        {}
        
		if( $check['i_id'] )
		{
			#Redirect!
			\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );	
		}		
		
		/* Key */
		$key = md5( $_SERVER['REMOTE_ADDR'] . ')(&^$#!%^&E$^*)' . uniqid( microtime(), true ) );
		
		/* Add log */				
        $log              = new \IPS\rsystem\Log;
        $log->time        = time();
        $log->inviter_id  = $reff;
        $log->secure_key  = $key; 
        $log->status      = 'sent';
        $log->times_sent  = 1;
        $log->invited_ip  = \IPS\Member::loggedIn()->ip_address;
        $log->save();

		/* Set cookie */
		\IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', $key, \IPS\DateTime::create()->add( new \DateInterval( 'P1Y') ), FALSE ); 
		
		/* Redirect! */
		\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );		 		
	}
}