<?php


namespace IPS\rsystem\modules\front\rsystem;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * main
 */
class _main extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Do we have refferal key inside URL? */
		if( isset( \IPS\Request::i()->reff ) && \IPS\Request::i()->reff )
		{
			  \IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=rsystem&module=rsystem&controller=main&do=checkReferral&reff=" . \IPS\Request::i()->reff, 'front', '' ) );
		}
		
		/* Do we have invite key inside URL? */
		if( isset( \IPS\Request::i()->key ) && \IPS\Request::i()->key )
		{
			 \IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=rsystem&module=rsystem&controller=invite&do=receiveValidate&key=" . \IPS\Request::i()->key, 'front', '' ) );
		}
				
		/* Redirect to the selected page */
		\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );
	}
    
	public function checkReferral()
	{
		/* INIT */		
		$_reff    = trim( \IPS\Request::i()->reff );
        $referrer = 0;
        $field    = \IPS\Settings::i()->dp41_rsystem_enable_mask ? 'dp3_rs_hashcode' : 'member_id'; 
        
        /* Get referral ID */
		try
        {
            $referrer = \IPS\Db::i()->select( 'member_id', 'core_members', array( $field . ' = ?', $_reff ) )->first(); 
        }
        catch ( \UnderflowException $e )
        {}
        
        if( ! $referrer )
        {
 			#Redirect!
			\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );	           
        }
        else
        {
            $reff = $referrer;
        }
        
		/* Invite only mode? */
		if ( \IPS\Settings::i()->dp41_rsystem_invite_type == 'invite' )
		{
			\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );		
		}			
		
		/* The same key as member ID */
		if( $reff == \IPS\Member::loggedIn()->member_id )
		{
            \IPS\Output::i()->error( 'rsystem_the_same_reff_key', 'RSYSTEM_019', 403, '' );
		}
		
		/* Logged member? */
		if( \IPS\Member::loggedIn()->member_id )
		{
            \IPS\Output::i()->error( 'rsystem_logged_already', 'RSYSTEM_020', 403, '' );			
		}
		
		/* Get exists cookie */
		if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_id'] ) )
        {
            $exists = \IPS\Request::i()->cookie['dp3_rs_invite_reff_id'];
		}
        else
        {
            $exists = '-';
        }
        
		/* Already referred? */
		if( \IPS\Member::loggedIn()->member_id && ( $reff == $exists ) )
		{
            \IPS\Output::i()->error( 'rsystem_referred_already', 'RSYSTEM_021', 403, '' );					
		}
		
		/* Check referrer */
		$referrer = \IPS\Member::load( $reff );
		
		/* Check allowed group */
		if( ! in_array( $referrer->member_group_id, explode( ',', \IPS\Settings::i()->dp41_rsystem_groups ) ) )
		{
			if( \IPS\Settings::i()->dp41_rsystem_groups != 'all' )
            {
                \IPS\Output::i()->error( 'rsystem_not_allowed_group', 'RSYSTEM_022', 403, '' );
            }
		}
		
		/* Check referred enabled */		
		if( ! $referrer->dp3_rs_menable )
		{
            \IPS\Output::i()->error( 'rsystem_not_enabled', 'RSYSTEM_023', 403, '' );
		}
		
		/* Check banned status */
		if( $referrer->dp3_rs_banned )
		{
            \IPS\Output::i()->error( 'rsystem_is_banned', 'RSYSTEM_024', 403, '' );
		}
		
		/* From mask but masks are disabled? */
		if( ! \IPS\Settings::i()->dp41_rsystem_enable_mask && \IPS\Request::i()->fromFurl )
		{
            \IPS\Output::i()->error( 'rsystem_masks_are_disabled', 'RSYSTEM_025', 403, '' );			
		}
				
        /* Work hard! */
		if( $reff )
		{
			/* Set cookie */
            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', $reff, \IPS\DateTime::create()->add( new \DateInterval( 'P1Y') ), FALSE );
			
            if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
            {
                $reff_cookie = \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'];
            }
            else
            {
                $reff_cookie = '-';
            }
			
            if( ! $reff_cookie || $reff_cookie == '-' )
			{			
				/* Key */				
				$key = md5( $_SERVER['REMOTE_ADDR'] . ')(&^$#!%^&E$^*)' . uniqid( microtime(), true ) );
				
				/* Add log */				
                $log              = new \IPS\rsystem\Log;
                $log->time        = time();
                $log->inviter_id  = $reff;
                $log->secure_key  = $key; 
                $log->status      = 'sent';
                $log->times_sent  = 1;
                $log->invited_ip  = \IPS\Member::loggedIn()->ip_address;
                $log->save();

				/* Set cookie */
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', $key, \IPS\DateTime::create()->add( new \DateInterval( 'P1Y') ), FALSE );
			}			
		}

		/* Redirect! */
		\IPS\Output::i()->redirect( \IPS\rsystem\Settings::whereToRedirect() );		
	}
}