<?php
 
namespace IPS\rsystem;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Log Model
 */
class _Log extends \IPS\Content\Item
{
	
	/**
	 * @brief	Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	Database Table
	 */
	public static $databaseTable = 'rsystem_referrals';
	
	/**
	 * @brief	Application
	 */
	public static $application = 'rsystem';
    
	/**
	 * @brief	Module
	 */
	public static $module = 'rsystem';    
	
	/**
	 * @brief	Database Prefix
	 */
	public static $databasePrefix = 'i_';
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
    
	/**
	 * @brief	Form Lang Prefix
	 */
	public static $formLangPrefix = 'rsystem_log_';     
    		
	/**
	 * @brief	Database Column Map
	 */
	public static $databaseColumnMap = array(
			'date'			 => 'time',
            'ip_address'     => 'invited_ip',
            'author'         => 'inviter_id',
            'content'        => 'body'
            
	);
	
	/**
	 * @brief	Title
	 */
	public static $title = 'rsystem_log';
	
    /**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'rsystem';
        
    /**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'i_time';
	
    /**
	 * @brief	[Node] Sortable?
	 */
	public static $nodeSortable = FALSE;
    
    /**
	 * @brief	[Node] Parent ID Database Column
	 */
	public static $databaseColumnParent = null;
    
	/**
	 * @brief	Title
	 */
	public static $icon = 'mail';

	/**
	 * @brief	Database ID Fields
	 */
	protected static $databaseIdFields = array( 'i_friend_mail' );
    
    /**
	 * [Node] Does the currently logged in user have permission to copy this node?
	 *
	 * @return	bool
	 */
	public function canCopy()
	{
        return false;
	}
	
    /**
	 * Set Default Values (overriding $defaultValues)
	 *
	 * @return	void
	 */
	protected function setDefaultValues()
	{
	   $this->time       = time();
       $this->invited_ip = \IPS\Request::i()->ipAddress();	
	} 
    
    
	/**
	 * Get elements for add/edit form
	 *
	 * @param	\IPS\Content\Item|NULL	$item		The current item if editing or NULL if creating
	 * @param	int						$container	Container (e.g. forum) ID, if appropriate
	 * @return	array
	 */
	public static function formElements( $item=NULL, \IPS\Node\Model $container=NULL )
	{
        $return = array();

        $return['log_form_invited_id'] = new \IPS\Helpers\Form\Member( 'log_form_invited_id', ( isset( $item->invited_id ) && $item->invited_id ) ? \IPS\Member::load( $item->invited_id )->name : ( ( isset( $item->user_pending ) && $item->user_pending ) ? \IPS\Member::load( $item->user_pending )->name : NULL), TRUE, array( 'disabled' => ( isset( $item->id ) && $item->invited_id ) ? TRUE : FALSE ) );
        $return['log_form_inviter_id'] = new \IPS\Helpers\Form\Member( 'log_form_inviter_id', ( isset( $item->inviter_id ) && $item->inviter_id ) ? \IPS\Member::load( $item->inviter_id )->name : NULL, TRUE );
        
		$return['log_form_add_points'] = new \IPS\Helpers\Form\YesNo( 'log_form_add_points', NULL, FALSE, array(), NULL, NULL, NULL, 'log_form_add_points' );
       
        return $return;
	}

	/**
	 * Process create/edit form
	 *
	 * @param	array				$values	Values from form
	 * @return	void
	 */
	public function processForm( $values )
	{  
        foreach( $values as $k => $v )
        {
            $key = str_replace( 'log_form_', '', $k );          
 
            if( $k == 'log_form_invited_id' )
            {
                $this->invited_id = $values['log_form_invited_id']->member_id;
                unset( $values[ $k ] );
            }
            elseif( $k == 'log_form_inviter_id' )
            {
                $this->inviter_id = $values['log_form_inviter_id']->member_id;
                 unset( $values[ $k ] );
            } 
            
            unset( $values['log_form_add_points'] );
        }

		parent::processForm( $values );
	}
    
    /**
	 * @brief	Cached URL
	 */
	protected $_url	= NULL;
    
	public function acpUrl()
	{
		return \IPS\Http\Url::internal( "app=rsystem&module=logs&controller=manage", 'admin' );
	}

	public function get__time()
	{
		return \IPS\DateTime::ts( $this->time );
	}
}